@@ -6,9 +6,6 @@ st_options(plain.ascii = FALSE,
            style = "rmarkdown",
            footnote = NA,
            subtitle.emphasis = FALSE)
-if (dir.exists("./img")) {
-  unlink("./img", recursive = TRUE)
-}
 
 ## ---- echo=FALSE---------------------------------------------------------
 st_css()
@@ -20,6 +17,10 @@ freq(iris$Species, plain.ascii = FALSE, style = "rmarkdown")
 ## ------------------------------------------------------------------------
 freq(iris$Species, report.nas = FALSE, headings = FALSE)
 
+## ------------------------------------------------------------------------
+freq(iris$Species, report.nas = FALSE, totals = FALSE,
+     cumul = FALSE, style = "rmarkdown", headings = FALSE)
+
 ## ---- eval=FALSE---------------------------------------------------------
 #  freq(tobacco[ ,c("gender", "age.gr", "smoker")])
 
@@ -31,6 +32,12 @@ with(tobacco,
      print(ctable(smoker, diseased, prop = 'n', totals = FALSE),
            headings = FALSE, method = "render"))
 
+## ------------------------------------------------------------------------
+library(magrittr)
+tobacco %$% 
+  ctable(gender, smoker, chisq = TRUE, headings = FALSE) %>%
+  print(method = "render")
+
 ## ------------------------------------------------------------------------
 descr(iris, style = "rmarkdown")
 
@@ -45,6 +52,11 @@ descr(iris, stats = "common", transpose = TRUE, headings = FALSE)
 #  dfSummary(tobacco, plain.ascii = FALSE, style = "grid",
 #            graph.magnif = 0.75, valid.col = FALSE, tmp.img.dir = "/tmp")
 
+## ---- results='markup'---------------------------------------------------
+library(magrittr)
+iris %>% descr(stats = "common") %>% tb()
+iris$Species %>% freq(cumul = FALSE, report.nas = FALSE) %>% tb()
+
 ## ------------------------------------------------------------------------
 (iris_stats_by_species <- stby(data = iris, 
                                INDICES = iris$Species, 
@@ -119,3 +131,6 @@ freq(iris$Species)
 Sys.setlocale("LC_CTYPE", "")
 st_options(lang = "en")
 
+## ---- eval=FALSE---------------------------------------------------------
+#  define_keywords(freq = "N")
+
